@@ -5,6 +5,7 @@ package raft
 
 import (
 	"bytes"
+	"context"
 	"fmt"
 	"net"
 	"reflect"
@@ -199,6 +200,31 @@ func TestNetworkTransport_Heartbeat_FastPath(t *testing.T) {
 	}
 }
 
+func makeAppendRPC() AppendEntriesRequest {
+	return AppendEntriesRequest{
+		Term:         10,
+		PrevLogEntry: 100,
+		PrevLogTerm:  4,
+		Entries: []*Log{
+			{
+				Index: 101,
+				Term:  4,
+				Type:  LogNoop,
+			},
+		},
+		LeaderCommitIndex: 90,
+		RPCHeader:         RPCHeader{Addr: []byte("cartman")},
+	}
+}
+
+func makeAppendRPCResponse() AppendEntriesResponse {
+	return AppendEntriesResponse{
+		Term:    4,
+		LastLog: 90,
+		Success: true,
+	}
+}
+
 func TestNetworkTransport_AppendEntries(t *testing.T) {
 
 	for _, useAddrProvider := range []bool{true, false} {
@@ -211,26 +237,8 @@ func TestNetworkTransport_AppendEntries(t *testing.T) {
 		rpcCh := trans1.Consumer()
 
 		// Make the RPC request
-		args := AppendEntriesRequest{
-			Term:         10,
-			PrevLogEntry: 100,
-			PrevLogTerm:  4,
-			Entries: []*Log{
-				{
-					Index: 101,
-					Term:  4,
-					Type:  LogNoop,
-				},
-			},
-			LeaderCommitIndex: 90,
-			RPCHeader:         RPCHeader{Addr: []byte("cartman")},
-		}
-
-		resp := AppendEntriesResponse{
-			Term:    4,
-			LastLog: 90,
-			Success: true,
-		}
+		args := makeAppendRPC()
+		resp := makeAppendRPCResponse()
 
 		// Listen for a request
 		go func() {
@@ -282,26 +290,8 @@ func TestNetworkTransport_AppendEntriesPipeline(t *testing.T) {
 		rpcCh := trans1.Consumer()
 
 		// Make the RPC request
-		args := AppendEntriesRequest{
-			Term:         10,
-			PrevLogEntry: 100,
-			PrevLogTerm:  4,
-			Entries: []*Log{
-				{
-					Index: 101,
-					Term:  4,
-					Type:  LogNoop,
-				},
-			},
-			LeaderCommitIndex: 90,
-			RPCHeader:         RPCHeader{Addr: []byte("cartman")},
-		}
-
-		resp := AppendEntriesResponse{
-			Term:    4,
-			LastLog: 90,
-			Success: true,
-		}
+		args := makeAppendRPC()
+		resp := makeAppendRPCResponse()
 
 		// Listen for a request
 		go func() {
@@ -368,26 +358,8 @@ func TestNetworkTransport_AppendEntriesPipeline_CloseStreams(t *testing.T) {
 	rpcCh := trans1.Consumer()
 
 	// Make the RPC request
-	args := AppendEntriesRequest{
-		Term:         10,
-		PrevLogEntry: 100,
-		PrevLogTerm:  4,
-		Entries: []*Log{
-			{
-				Index: 101,
-				Term:  4,
-				Type:  LogNoop,
-			},
-		},
-		LeaderCommitIndex: 90,
-		RPCHeader:         RPCHeader{Addr: []byte("cartman")},
-	}
-
-	resp := AppendEntriesResponse{
-		Term:    4,
-		LastLog: 90,
-		Success: true,
-	}
+	args := makeAppendRPC()
+	resp := makeAppendRPCResponse()
 
 	shutdownCh := make(chan struct{})
 	defer close(shutdownCh)
@@ -467,6 +439,105 @@ func TestNetworkTransport_AppendEntriesPipeline_CloseStreams(t *testing.T) {
 	}
 }
 
+func TestNetworkTransport_AppendEntriesPipeline_MaxRPCsInFlight(t *testing.T) {
+	// Test the important cases 0 (default to 2), 1 (disabled), 2 and "some"
+	for _, max := range []int{0, 1, 2, 10} {
+		t.Run(fmt.Sprintf("max=%d", max), func(t *testing.T) {
+			config := &NetworkTransportConfig{
+				MaxPool:         2,
+				MaxRPCsInFlight: max,
+				Timeout:         time.Second,
+				// Don't use test logger as the transport has multiple goroutines and
+				// causes panics.
+				ServerAddressProvider: &testAddrProvider{"localhost:0"},
+			}
+
+			// Transport 1 is consumer
+			trans1, err := NewTCPTransportWithConfig("localhost:0", nil, config)
+			require.NoError(t, err)
+			defer trans1.Close()
+
+			// Make the RPC request
+			args := makeAppendRPC()
+			resp := makeAppendRPCResponse()
+
+			ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
+			defer cancel()
+
+			// Transport 2 makes outbound request
+			config.ServerAddressProvider = &testAddrProvider{string(trans1.LocalAddr())}
+			trans2, err := NewTCPTransportWithConfig("localhost:0", nil, config)
+			require.NoError(t, err)
+			defer trans2.Close()
+
+			// Kill the transports on the timeout to unblock. That means things that
+			// shouldn't have blocked did block.
+			go func() {
+				<-ctx.Done()
+				trans2.Close()
+				trans1.Close()
+			}()
+
+			// Attempt to pipeline
+			pipeline, err := trans2.AppendEntriesPipeline("id1", trans1.LocalAddr())
+			if max == 1 {
+				// Max == 1 implies no pipelining
+				require.EqualError(t, err, ErrPipelineReplicationNotSupported.Error())
+				return
+			}
+			require.NoError(t, err)
+
+			expectedMax := max
+			if max == 0 {
+				// Should have defaulted to 2
+				expectedMax = 2
+			}
+
+			for i := 0; i < expectedMax-1; i++ {
+				// We should be able to send `max - 1` rpcs before `AppendEntries`
+				// blocks. It blocks on the `max` one because it it sends before pushing
+				// to the chan. It will block forever when it does because nothing is
+				// responding yet.
+				out := new(AppendEntriesResponse)
+				_, err := pipeline.AppendEntries(&args, out)
+				require.NoError(t, err)
+			}
+
+			// Verify the next send blocks without blocking test forever
+			errCh := make(chan error, 1)
+			go func() {
+				out := new(AppendEntriesResponse)
+				_, err := pipeline.AppendEntries(&args, out)
+				errCh <- err
+			}()
+
+			select {
+			case err := <-errCh:
+				require.NoError(t, err)
+				t.Fatalf("AppendEntries didn't block with %d in flight", max)
+			case <-time.After(50 * time.Millisecond):
+				// OK it's probably blocked or we got _really_ unlucky with scheduling!
+			}
+
+			// Verify that once we receive/respond another one can be sent.
+			rpc := <-trans1.Consumer()
+			rpc.Respond(resp, nil)
+
+			// We also need to consume the response from the pipeline in case chan is
+			// unbuffered (inflight is 2 or 1)
+			<-pipeline.Consumer()
+
+			// The last append should unblock once the response is received.
+			select {
+			case <-errCh:
+				// OK
+			case <-time.After(50 * time.Millisecond):
+				t.Fatalf("last append didn't unblock")
+			}
+		})
+	}
+}
+
 func TestNetworkTransport_RequestVote(t *testing.T) {
 
 	for _, useAddrProvider := range []bool{true, false} {
@@ -741,11 +812,18 @@ func TestNetworkTransport_PooledConn(t *testing.T) {
 }
 
 func makeTransport(t *testing.T, useAddrProvider bool, addressOverride string) (*NetworkTransport, error) {
-	if useAddrProvider {
-		config := &NetworkTransportConfig{MaxPool: 2, Timeout: time.Second, Logger: newTestLogger(t), ServerAddressProvider: &testAddrProvider{addressOverride}}
-		return NewTCPTransportWithConfig("localhost:0", nil, config)
+	config := &NetworkTransportConfig{
+		MaxPool: 2,
+		// Setting this because older tests for pipelining were written when this
+		// was a constant and block forever if it's not large enough.
+		MaxRPCsInFlight: 130,
+		Timeout:         time.Second,
+		Logger:          newTestLogger(t),
 	}
-	return NewTCPTransportWithLogger("localhost:0", nil, 2, time.Second, newTestLogger(t))
+	if useAddrProvider {
+		config.ServerAddressProvider = &testAddrProvider{addressOverride}
+	}
+	return NewTCPTransportWithConfig("localhost:0", nil, config)
 }
 
 type testCountingWriter struct {
