@@ -4,12 +4,10 @@
 package raft
 
 import (
-	"bufio"
 	"bytes"
+	"encoding/json"
 	"errors"
 	"fmt"
-	"io/ioutil"
-	"log"
 	"os"
 	"path/filepath"
 	"reflect"
@@ -428,12 +426,9 @@ func TestRaft_LeaderFail(t *testing.T) {
 		if len(fsm.logs) != 2 {
 			t.Fatalf("did not apply both to FSM! %v", fsm.logs)
 		}
-		if bytes.Compare(fsm.logs[0], []byte("test")) != 0 {
-			t.Fatalf("first entry should be 'test'")
-		}
-		if bytes.Compare(fsm.logs[1], []byte("apply")) != 0 {
-			t.Fatalf("second entry should be 'apply'")
-		}
+
+		require.Equal(t, fsm.logs[0], []byte("test"))
+		require.Equal(t, fsm.logs[1], []byte("apply"))
 		fsm.Unlock()
 	}
 }
@@ -1154,9 +1149,10 @@ func TestRaft_SnapshotRestore_Progress(t *testing.T) {
 	// Intercept logs and look for specific log messages.
 	var logbuf lockedBytesBuffer
 	cfg.Logger = hclog.New(&hclog.LoggerOptions{
-		Name:   "test",
-		Level:  hclog.Info,
-		Output: &logbuf,
+		Name:       "test",
+		JSONFormat: true,
+		Level:      hclog.Info,
+		Output:     &logbuf,
 	})
 	r, err := NewRaft(&cfg, r.fsm, r.logs, r.stable, r.snapshots, trans2)
 	if err != nil {
@@ -1170,14 +1166,26 @@ func TestRaft_SnapshotRestore_Progress(t *testing.T) {
 	}
 
 	{
-		scan := bufio.NewScanner(strings.NewReader(logbuf.String()))
+		dec := json.NewDecoder(strings.NewReader(logbuf.String()))
+
 		found := false
-		for scan.Scan() {
-			line := scan.Text()
-			if strings.Contains(line, "snapshot restore progress") && strings.Contains(line, "percent-complete=100.00%") {
+
+		type partialRecord struct {
+			Message         string `json:"@message"`
+			PercentComplete string `json:"percent-complete"`
+		}
+
+		for !found {
+			var record partialRecord
+			if err := dec.Decode(&record); err != nil {
+				t.Fatalf("error while decoding json logs: %v", err)
+			}
+
+			if record.Message == "snapshot restore progress" && record.PercentComplete == "100.00%" {
 				found = true
 				break
 			}
+
 		}
 		if !found {
 			t.Fatalf("could not find a log line indicating that snapshot restore progress was being logged")
@@ -1309,13 +1317,13 @@ func TestRaft_SnapshotRestore_PeerChange(t *testing.T) {
 	content := []byte(fmt.Sprintf("[%s]", strings.Join(peers, ",")))
 
 	// Perform a manual recovery on the cluster.
-	base, err := ioutil.TempDir("", "")
+	base, err := os.MkdirTemp("", "")
 	if err != nil {
 		t.Fatalf("err: %v", err)
 	}
 	defer os.RemoveAll(base)
 	peersFile := filepath.Join(base, "peers.json")
-	if err = ioutil.WriteFile(peersFile, content, 0666); err != nil {
+	if err = os.WriteFile(peersFile, content, 0666); err != nil {
 		t.Fatalf("[ERR] err: %v", err)
 	}
 	configuration, err := ReadPeersJSON(peersFile)
@@ -1549,9 +1557,7 @@ func snapshotAndRestore(t *testing.T, offset uint64, monotonicLogStore bool, res
 	}
 	for i, entry := range fsm.logs {
 		expected := []byte(fmt.Sprintf("test %d", i))
-		if !bytes.Equal(entry, expected) {
-			t.Fatalf("Log entry bad: %v", entry)
-		}
+		require.Equal(t, entry, expected)
 	}
 	fsm.Unlock()
 
@@ -2921,8 +2927,8 @@ func TestRaft_FollowerRemovalNoElection(t *testing.T) {
 	c := MakeCluster(3, t, inmemConf)
 
 	defer c.Close()
-	waitForLeader(c)
-
+	err := waitForLeader(c)
+	require.NoError(t, err)
 	leader := c.Leader()
 
 	// Wait until we have 2 followers
@@ -2975,8 +2981,8 @@ func TestRaft_VoteWithNoIDNoAddr(t *testing.T) {
 	c := MakeCluster(3, t, nil)
 
 	defer c.Close()
-	waitForLeader(c)
-
+	err := waitForLeader(c)
+	require.NoError(t, err)
 	leader := c.Leader()
 
 	// Wait until we have 2 followers
@@ -3045,25 +3051,6 @@ func waitForLeader(c *cluster) error {
 	return errors.New("no leader elected")
 }
 
-func waitForNewLeader(c *cluster, id ServerID) error {
-	count := 0
-	for count < 100 {
-		r := c.GetInState(Leader)
-		if len(r) >= 1 && r[0].localID != id {
-			return nil
-		} else {
-			if len(r) == 0 {
-				log.Println("no leader yet")
-			} else {
-				log.Printf("leader still %s\n", id)
-			}
-		}
-		count++
-		time.Sleep(100 * time.Millisecond)
-	}
-	return errors.New("no leader elected")
-}
-
 func TestRaft_runFollower_State_Transition(t *testing.T) {
 	type fields struct {
 		conf     *Config
